/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import expect from '@kbn/expect';
import { Datafeed } from '@kbn/ml-plugin/common/types/anomaly_detection_jobs';
import type { MlAnomalyCategorizerStatsDoc } from '@kbn/ml-anomaly-utils';
import { FtrProviderContext } from '../../../ftr_provider_context';
import { USER } from '../../../../functional/services/ml/security_common';
import { getCommonRequestHeader } from '../../../../functional/services/ml/common_api';

export default ({ getService }: FtrProviderContext) => {
  const esArchiver = getService('esArchiver');
  const supertest = getService('supertestWithoutAuth');
  const ml = getService('ml');

  const jobId = `sample_logs_${Date.now()}`;
  const PARTITION_FIELD_NAME = 'event.dataset';
  const testJobConfig = {
    job_id: jobId,
    groups: ['sample_logs', 'bootstrap', 'categorization'],
    description: "count by mlcategory (message) on 'sample logs' dataset with 15m bucket span",
    analysis_config: {
      bucket_span: '15m',
      categorization_field_name: 'message',
      per_partition_categorization: { enabled: true, stop_on_warn: true },
      detectors: [
        {
          function: 'count',
          by_field_name: 'mlcategory',
          partition_field_name: PARTITION_FIELD_NAME,
        },
      ],
      influencers: ['mlcategory'],
    },
    analysis_limits: { model_memory_limit: '26MB' },
    data_description: { time_field: '@timestamp', time_format: 'epoch_ms' },
    model_plot_config: { enabled: false, annotations_enabled: true },
    model_snapshot_retention_days: 10,
    daily_model_snapshot_retention_after_days: 1,
    allow_lazy_open: false,
  };
  // @ts-expect-error not full interface
  const testDatafeedConfig: Datafeed = {
    datafeed_id: `datafeed-${jobId}`,
    indices: ['ft_module_sample_logs'],
    job_id: jobId,
    query: { bool: { must: [{ match_all: {} }] } },
  };

  describe('get categorizer_stats', function () {
    before(async () => {
      await esArchiver.loadIfNeeded('x-pack/test/functional/es_archives/ml/module_sample_logs');
      await ml.testResources.setKibanaTimeZoneToUTC();
      // @ts-expect-error not full interface
      await ml.api.createAndRunAnomalyDetectionLookbackJob(testJobConfig, testDatafeedConfig);
    });

    after(async () => {
      await ml.testResources.deleteDataViewByTitle('ft_module_sample_logs');
      await ml.api.cleanMlIndices();
    });

    it('should fetch all the categorizer stats for job id', async () => {
      const { body, status } = await supertest
        .get(`/internal/ml/results/${jobId}/categorizer_stats`)
        .auth(USER.ML_POWERUSER, ml.securityCommon.getPasswordForUser(USER.ML_POWERUSER))
        .set(getCommonRequestHeader('1'));
      ml.api.assertResponseStatusCode(200, status, body);

      body.forEach((doc: MlAnomalyCategorizerStatsDoc) => {
        expect(doc.job_id).to.eql(jobId);
        expect(doc.result_type).to.eql('categorizer_stats');
        expect(doc.partition_field_name).to.be(PARTITION_FIELD_NAME);
        expect(doc.partition_field_value).to.not.be(undefined);
      });
    });

    it('should fetch categorizer stats for job id for user with view permission', async () => {
      const { body, status } = await supertest
        .get(`/internal/ml/results/${jobId}/categorizer_stats`)
        .auth(USER.ML_VIEWER, ml.securityCommon.getPasswordForUser(USER.ML_VIEWER))
        .set(getCommonRequestHeader('1'));
      ml.api.assertResponseStatusCode(200, status, body);

      body.forEach((doc: MlAnomalyCategorizerStatsDoc) => {
        expect(doc.job_id).to.eql(jobId);
        expect(doc.result_type).to.eql('categorizer_stats');
        expect(doc.partition_field_name).to.be(PARTITION_FIELD_NAME);
        expect(doc.partition_field_value).to.not.be(undefined);
      });
    });

    it('should not fetch categorizer stats for job id for unauthorized user', async () => {
      const { body, status } = await supertest
        .get(`/internal/ml/results/${jobId}/categorizer_stats`)
        .auth(USER.ML_UNAUTHORIZED, ml.securityCommon.getPasswordForUser(USER.ML_UNAUTHORIZED))
        .set(getCommonRequestHeader('1'));
      ml.api.assertResponseStatusCode(403, status, body);

      expect(body.error).to.be('Forbidden');
    });

    it('should fetch all the categorizer stats with per-partition value for job id', async () => {
      const { body, status } = await supertest
        .get(`/internal/ml/results/${jobId}/categorizer_stats`)
        .query({ partitionByValue: 'sample_web_logs' })
        .auth(USER.ML_POWERUSER, ml.securityCommon.getPasswordForUser(USER.ML_POWERUSER))
        .set(getCommonRequestHeader('1'));
      ml.api.assertResponseStatusCode(200, status, body);

      body.forEach((doc: MlAnomalyCategorizerStatsDoc) => {
        expect(doc.job_id).to.eql(jobId);
        expect(doc.result_type).to.eql('categorizer_stats');
        expect(doc.partition_field_name).to.be(PARTITION_FIELD_NAME);
        expect(doc.partition_field_value).to.be('sample_web_logs');
      });
    });

    it('should fetch categorizer stats with per-partition value for user with view permission', async () => {
      const { body, status } = await supertest
        .get(`/internal/ml/results/${jobId}/categorizer_stats`)
        .query({ partitionByValue: 'sample_web_logs' })
        .auth(USER.ML_VIEWER, ml.securityCommon.getPasswordForUser(USER.ML_VIEWER))
        .set(getCommonRequestHeader('1'));
      ml.api.assertResponseStatusCode(200, status, body);

      body.forEach((doc: MlAnomalyCategorizerStatsDoc) => {
        expect(doc.job_id).to.eql(jobId);
        expect(doc.result_type).to.eql('categorizer_stats');
        expect(doc.partition_field_name).to.be(PARTITION_FIELD_NAME);
        expect(doc.partition_field_value).to.be('sample_web_logs');
      });
    });

    it('should not fetch categorizer stats with per-partition value for unauthorized user', async () => {
      const { body, status } = await supertest
        .get(`/internal/ml/results/${jobId}/categorizer_stats`)
        .query({ partitionByValue: 'sample_web_logs' })
        .auth(USER.ML_UNAUTHORIZED, ml.securityCommon.getPasswordForUser(USER.ML_UNAUTHORIZED))
        .set(getCommonRequestHeader('1'));
      ml.api.assertResponseStatusCode(403, status, body);

      expect(body.error).to.be('Forbidden');
    });
  });
};
